class LSystemClass {
  // Initial axiom and production rules
  String axiom = "F";
  String[][] rules = {
    {"F", "X"},
    {"+", "+"},
    {"-", "-"},
    {"[", "["},
    {"]", "]"},
    {"C", "C"},
    {"G", "G"},
    {"X", "FF[+FXC]FF[-FXC]"}
  };

  String computedWord;
  int iterations;
  float size, angle, scaleFactor;

  LSystemClass(float size, float angle, float scaleFactor) {
    this.size = size;
    this.angle = angle;
    this.scaleFactor = scaleFactor;
    computedWord = axiom;
    iterations = 0;
    background(255); // Set the background color
  }

  void iteration() {
    char s;
    String newWord = "";
    size *= scaleFactor;
    for (int i = 0; i < computedWord.length(); i++) {
      s = computedWord.charAt(i);
      boolean ruleApplied = false;
      for (int j = 0; j < rules.length; j++) {
        if (s == rules[j][0].charAt(0)) {
          newWord += rules[j][1]; // Apply the production rule
          ruleApplied = true;
          break;
        }
      }
      if (!ruleApplied) {
        newWord += s; // Keep unchanged characters
      }
    }
    iterations++;
    computedWord = newWord;
    println(iterations + ": " + computedWord);
    if (iterations == 7) {
      noLoop(); // Stop further iterations after a certain count
    }
  }

  void drawLSystem(Turtle t) {
    char s;
    for (int i = 0; i < computedWord.length(); i++) {
      s = computedWord.charAt(i);
      switch (s) {
        case 'F':
          t.forward(size); // Move forward by a certain distance
          break;
        case '+':
          t.right(angle); // Turn right by a certain angle
          break;
        case '-':
          t.left(angle); // Turn left by a certain angle
          break;
        case '[':
          t.push(); // Push the current state onto the stack
          break;
        case ']':
          t.pop(); // Pop the current state from the stack
          break;
        case 'C':
          randomChoice(); // Apply a random choice for 'C'
          break;
        default:
          break;
      }
    }
  }

  void leaf(Turtle t) {
    // Draw a leaf-like shape at the "turtle's" current position and heading
    float x = t.getX();
    float y = t.getY();
    float leafWidth = 10; // Adjust the leaf's width
    float leafHeight = 12; // Adjust the leaf's height
    fill(192, 128, 255); // Light purple fill color
    stroke(0); // Black stroke color

    // Draw the main elliptical part of the leaf
    ellipse(x, y, leafWidth, leafHeight);

    // Draw two additional ellipses to create the leaf's shape
    ellipse(x - leafWidth / 3, y - leafHeight / 2, leafWidth, leafHeight * 1.5);
    ellipse(x + leafWidth / 3, y - leafHeight / 2, leafWidth, leafHeight * 1.5);
  }

  void drawTriangleWithTurtle(Turtle t, float sideLength, int levels) {
    for (int i = 0; i < levels; i++) {
      float x1 = t.getX();
      float y1 = t.getY();
      t.forward(sideLength);
      float x2 = t.getX();
      float y2 = t.getY();
      t.left(120);
      t.forward(sideLength);
      float x3 = t.getX();
      float y3 = t.getY();
      t.left(120);

      // Draw the triangle using the current turtle position
      beginShape();
      vertex(x1, y1);
      vertex(x2, y2);
      vertex(x3, y3);
      endShape(CLOSE);

      // Move to the top of the current triangle for the next iteration
      t.penUp();
      t.forward(sideLength); // Move up
      t.penDown();

      sideLength /= 2; // Reduce the size for the next level
    }
  }

  void randomChoice() {
    // Generate a random float value between 0 and 1
    float randValue = random(1);

    if (randValue < 0.33) {
      // Option 1: Call the leaf function
      leaf(t);
    } else if (randValue < 0.67) {
      // Option 2: Draw an ellipse with a light purple fill
      fill(192, 128, 255); // Light purple fill color
      ellipse(t.getX(), t.getY(), 14, 6); // Draw the ellipse
    } else {
      fill(0, 128, 0); // Green fill color for the leaf
      drawTriangleWithTurtle(t, 13, 3);
    }
  }
}
