class LSystemClass {
  String axiom = "F";
  String[][] rules = {
    {"F", "X"},
    {"+", "+"},
    {"-", "-"},
    {"[", "["},
    {"]", "]"},
    {"C", "C"},
    {"G", "G"},
    {"X", "F[+F[---FC]C]F[-F[+++FC]C]"},
    //{"X", "FF-[-F+F+FC]+[+F-F-FC]"}
  };

  String computedWord;
  int iterations;
  float size, angle, scaleFactor;

  LSystemClass(float size, float angle, float scaleFactor) {
    this.size = size;
    this.angle = angle;
    this.scaleFactor = scaleFactor;
    computedWord = axiom;
    iterations = 0;
  }

  void iteration() {
    char s;
    String newWord = "";
    size *= scaleFactor;
    for (int i = 0; i < computedWord.length(); i++) {
      s = computedWord.charAt(i);
      boolean ruleApplied = false;
      for (int j = 0; j < rules.length; j++) {
        if (s == rules[j][0].charAt(0)) {
          newWord = newWord + rules[j][1];
          ruleApplied = true;
          break;
        }
      }
      if (!ruleApplied) {
        newWord = newWord + s;
      }
    }
    iterations++;
    computedWord = newWord;
    println(iterations + ": " + computedWord);
    if (iterations == 5) { // Adjust the number of iterations as needed
      //noLoop();
      // save("my_sketch.jpg"); // Save the sketch as "my_sketch.jpg"
    }
  }

  float getRandomSize(float minSize, float maxSize) {
    return random(minSize, maxSize);
  }

  float getRandomAngle(float minAngle, float maxAngle) {
    return random(minAngle, maxAngle);
  }

  void drawLSystem(Turtle t) {
    char s;
    for (int i = 0; i < computedWord.length(); i++) {
      s = computedWord.charAt(i);
      switch (s) {
        case 'F':
          t.forward(size);
          break;
        case '+':
          t.right(angle);
          break;
        case '-':
          t.left(angle);
          break;
        case '[':
          t.push();
          break;
        case ']':
          t.pop();
          break;
        case 'C':
          randomChoice(t);
          break;
        default:
          break;
      }
    }
  }

  void randomChoice(Turtle t) {
    float randValue = random(1);
      fill(0, 128, 0); // Green fill color for the leaf
    drawTriangleWithTurtle(t, 13, 5);
  }

  void leaf(Turtle t) {
    // Draw a leaf-like shape at the "turtle's" current position and heading
    float x = t.getX();
    float y = t.getY();
    float leafWidth = 5; // Adjust the leaf's width
    float leafHeight = 12; // Adjust the leaf's height
    //fill(0, 128, 0); // Green fill color for the leaf
    //fill(192, 128, 255); // Light purple fill color

    stroke(0); // Black stroke color

    // Draw the main elliptical part of the leaf
    ellipse(x, y, leafWidth, leafHeight);

    // Draw two additional ellipses to create the leaf's shape
    ellipse(x - leafWidth / 3, y - leafHeight / 2, leafWidth, leafHeight * 1.5);
    ellipse(x + leafWidth / 3, y - leafHeight / 2, leafWidth, leafHeight * 1.5);
  }

  void drawTriangleWithTurtle(Turtle t, float sideLength, int levels) {
    for (int i = 0; i < levels; i++) {
      float x1 = t.getX();
      float y1 = t.getY();
      t.forward(sideLength);
      float x2 = t.getX();
      float y2 = t.getY();
      t.left(120);
      t.forward(sideLength);
      float x3 = t.getX();
      float y3 = t.getY();
      t.left(120);

      // Draw the triangle using the current turtle position
      beginShape();
      vertex(x1, y1);
      vertex(x2, y2);
      vertex(x3, y3);
      endShape(CLOSE);

      // Move to the top of the current triangle for the next iteration
      t.penUp();
      t.forward(sideLength); // Move up
      t.penDown();

      sideLength /= 2; // Reduce the size for the next level
    }
  }
}
