class LSystemClass {
  String axiom = "X"; // Change the axiom to start with a different symbol
  String[][] rules = {
    {"X", "[-FXPS]F[+FXCPS]F"}, // Define new production rules
    {"F", "FF"} // Define a simple growth rule for 'F'
  };

  String computedWord;
  int iterations;
  float size, angle, scaleFactor;

  LSystemClass(float size, float angle, float scaleFactor) {
    this.size = size;
    this.angle = angle;
    this.scaleFactor = scaleFactor;
    computedWord = axiom;
    iterations = 0;
    background(#43A57D); // Set the background color
  }

  void iteration() {
    char s;
    String newWord = "";
    size *= scaleFactor;
    for (int i = 0; i < computedWord.length(); i++) {
      s = computedWord.charAt(i);
      boolean ruleApplied = false;
      for (int j = 0; j < rules.length; j++) {
        if (s == rules[j][0].charAt(0)) {
          newWord += rules[j][1]; // Apply the production rule
          ruleApplied = true;
          break;
        }
      }
      if (!ruleApplied) {
        newWord += s; // Keep unchanged characters
      }
    }
    iterations++;
    computedWord = newWord;
    println(iterations + ": " + computedWord);
    //if (iterations == 10) { // Adjust the number of iterations as needed
    //  noLoop(); // Stop further iterations after a certain count
    //}
  }
  
  
void drawThreeLeafPetal(Turtle t) {
  float petalWidth = 40; // Adjust the width of the petal
  float petalHeight = 10; // Adjust the height of the petal
  float curveFactor = 0.6; // Adjust the curve factor of the petal
  float spacing = 20; // Adjust the spacing between petals

  t.penUp(); // Lift the pen to move without drawing
  t.forward(petalWidth / 2); // Move to the center of the first petal
  t.left(90); // Turn the turtle to draw the petal vertically

  t.penDown(); // Lower the pen to start drawing
  for (int i = 0; i < 3; i++) {
    // Draw one of the three leaf petals using ellipses
    float x = t.getX();
    float y = t.getY();
    float ellipseWidth = petalWidth;
    float ellipseHeight = petalHeight;

    // Draw the petal as an ellipse
    fill(255, 165, 0); // Orange
    ellipse(x, y, ellipseWidth, ellipseHeight);

    t.penUp(); // Lift the pen
    t.forward(spacing); // Move forward to create spacing between petals
    t.penDown(); // Lower the pen

    t.left(120); // Turn the turtle for the next petal
  }
  t.penUp(); // Lift the pen to finish drawing
}


void drawGreenFilledTriangleLeaf(Turtle t, float leafSize) {
  t.penUp(); // Lift the pen to move without drawing
  t.left(90); // Turn the turtle to point upward
  t.forward(leafSize / 2); // Move to the base of the leaf
  t.right(90); // Turn the turtle to draw the leaf vertically
  t.penDown(); // Lower the pen to start drawing

  fill(0, 128, 0); // Green fill color
  stroke(0, 128, 0); // Green stroke color

  // Draw the green-filled triangle leaf
  for (int i = 0; i < 3; i++) {
    t.forward(leafSize); // Draw one side of the triangle
    t.left(120); // Turn the turtle for the next side
  }

  t.penUp(); // Lift the pen to finish drawing the leaf
}








void drawLSystem(Turtle t) {
  char s;
  for (int i = 0; i < computedWord.length(); i++) {
    s = computedWord.charAt(i);
    switch (s) {
      case 'F':
        t.forward(size); // Move forward by a certain distance
        break;
      case '+':
        t.right(angle); // Turn right by a certain angle
        break;
      case '-':
        t.left(angle); // Turn left by a certain angle
        break;
      case '[':
        t.push(); // Push the current state onto the stack
        break;
      case ']':
        t.pop(); // Pop the current state from the stack
        break;
      case 'P':
        drawThreeLeafPetal(t);
        break;
      case 'S':
        drawGreenFilledTriangleLeaf(t, 40); // Specify the leaf size (adjust as needed)
        break;
      default:
        break;
    }
  }
}

  
  
  
  
  
}
